// WSAInterface.cpp : This file contains the functions used to interface with the wsa via c++ DLL file.
//
#include "stdafx.h"
#include "WSAInterface.h"
#include "stdio.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

extern "C" {
	#include "thinkrf_stdint.h"
	#include "wsa_error.h"
	#include "wsa_commons.h"
	#include "wsa_lib.h"
	#include "wsa_api.h"
	#include "wsa_client.h"
	#include "wsa_sweep_device.h"
	#include <windows.h>
}
// keep track wheter this is an R5500
int R5500_UNIT = 0;

// declare local functions
void wsa_dev_init(int32_t cmd_sock, int32_t data_sock,  struct wsa_device *dev);

void copy_iq_data(int16_t *i16_buffer,
				  int16_t *q16_buffer, 
				  int32_t *i32_buffer, 
				  int16_t i16_data[MAX_SAMPLES_PER_PACKET], 
				  int16_t q16_data[MAX_SAMPLES_PER_PACKET],
				  int32_t i32_data[MAX_SAMPLES_PER_PACKET], 
				  uint32_t stream_id, 
				  int32_t sample_size);

void parse_wsa_sockets(int64_t wsa_handle, int32_t *cmd_sock, int32_t *data_sock);

// ////////////////////////////////////////////////////////////////////////////
// WSA RELATED FUNCTIONS                                                     //
// ////////////////////////////////////////////////////////////////////////////
/**
 * Establishes a connection d to the WSA. At success, the handle 
 * remains open for future access by other 
 *
 * @param wsa_handle - A pointer to a 64-bit integer which holds the socket info
 * @param ip - a char pointer to the wsa device
 * @return 0 on success, or a negative number on error.
 */
__declspec(dllexport) int16_t __stdcall  wsaConnect(int64_t *wsa_handle, char *ip) 
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev = &wsa_dev;
	char intf_str[250];
	int16_t result;

	sprintf_s(intf_str, "TCPIP::%s", ip);
	//result = wsa_ping(dev, intf_str);
	//if (result < 0)
	//	return result;

	result = wsa_open(dev, intf_str);
	if ((strstr(dev->descr.prod_model, R5500) != NULL))
		R5500_UNIT = 1;

	*wsa_handle =  ((int64_t) dev->sock.cmd << 32) + (int64_t) dev->sock.data;
	return  (int32_t) result;
}

/**
 * Close an already established connection d to the WSA.
 *
 * @param wsa_handle - A pointer to a 64-bit integer which holds the socket info
 * @return 0 on success, or a negative number on error.
 */
__declspec(dllexport) int16_t __stdcall  wsaDisconnect(int64_t wsa_handle) 
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev = &wsa_dev;
	int32_t data_sock;
	int32_t cmd_sock;
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	dev->sock.cmd = cmd_sock;
	dev->sock.data = data_sock;
	strcpy_s(dev->descr.intf_type, "TCPIP");
	wsa_dev_init( cmd_sock, data_sock, dev);
	
	wsa_close(dev);

	return 0;
}


/**
 * Reads one IF data packet, with all the corresponding context packets
 * @param wsa_handle - A pointer to a 64-bit integer which holds the socket info
 * @param i16_data - A pointer to a 16-bit buffer to hold the 16-bit idata
 * @param q16_data - A pointer to a 16-bit buffer to hold the 16-bit qdata
 * @param i32_data - A pointer to a 32-bit buffer to hold the 32-bit idata
 * @param timeout - A value used to set the socket read timeout (Milliseconds)
 * @param stream_id - A pointer to store the IF data packet's stream id
 * @param spectral_inversion - A pointer to to store the spectral inversion state
 * @param samples_per_packet - A pointer to store the number of samples
 * @param timestamp_sec - A pointer to hold the timestamp value (seconds)
 * @param timestamp_psec - A pointer to hold the timestamp value ( pico seconds)
 * @param reference_level - A pointer to store reference level to calibrate the
 * spectral data (dBm)
 * @param bandwidth - The bandwidth of the IF data (Hz)
 * @param center_frequency - The center frequency of the IF data (Hz)
 * @return 0 on success or a negative value on error
 */
__declspec(dllexport) int16_t __stdcall wsaReadData(int64_t wsa_handle,
												int16_t *i16_data, 
												int16_t *q16_data,
												int32_t *i32_data,
												uint32_t timeout,
												uint32_t *stream_id,
												uint8_t *spectral_inversion,
												int32_t *samples_per_packet,
												uint32_t *timestamp_sec,
												uint64_t *timestamp_psec,
												int16_t *reference_level,
												int64_t *bandwidth,
												int64_t *center_frequency)
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev = &wsa_dev;
	int8_t exit_loop = 0;
	int16_t result = 0;
	struct wsa_vrt_packet_header *header;
	struct wsa_vrt_packet_trailer *trailer;
	struct wsa_receiver_packet *receiver;
	struct wsa_digitizer_packet *digitizer;
	struct wsa_extension_packet *extension;
	int32_t data_sock;
	int32_t cmd_sock;
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init(cmd_sock, data_sock, dev);
	
	// Allocate header buffer space
	header = (struct wsa_vrt_packet_header *) malloc(sizeof(struct wsa_vrt_packet_header));
	if (header == NULL)	
		return WSA_ERR_MALLOCFAILED;

	// Allocate trailer buffer space
	trailer = (struct wsa_vrt_packet_trailer *) malloc(sizeof(struct wsa_vrt_packet_trailer));
	if (trailer == NULL)
	{
		free(header);
		return WSA_ERR_MALLOCFAILED;
	}

	// allocate receiver packet buffer space
	receiver = (struct wsa_receiver_packet *) malloc(sizeof(struct wsa_receiver_packet));
	if (receiver == NULL)
	{
		free(header);
		free(trailer);
		
		return WSA_ERR_MALLOCFAILED;
	}

	// allocate digitizer packet buffer space
	digitizer = (struct wsa_digitizer_packet *) malloc(sizeof(struct wsa_digitizer_packet));
	if (digitizer == NULL)
	{
		free(header);
		free(trailer);
		free(receiver);
		
		return WSA_ERR_MALLOCFAILED;
	}

	// allocate extension packet buffer space
	extension = (struct wsa_extension_packet *) malloc(sizeof(struct wsa_extension_packet));
	if (extension == NULL)
	{
		free(header);
		free(trailer);
		free(receiver);
		free(digitizer);

		return WSA_ERR_MALLOCFAILED;
	}

	while (exit_loop != 1)
	{
		result = wsa_read_vrt_packet(dev, header, trailer, receiver, digitizer,
							extension, i16_data, q16_data, i32_data, *samples_per_packet, timeout);
		if (result < 0)
			return result;

		if (header->packet_type == IF_PACKET_TYPE)
		{
			exit_loop = 1;
			*timestamp_sec = header->time_stamp.sec;
			*timestamp_psec = header->time_stamp.psec;
			*stream_id = header->stream_id;
			*spectral_inversion = trailer->spectral_inversion_indicator;
			*reference_level = digitizer->reference_level;			
			*bandwidth = (int64_t) digitizer->bandwidth;
			*center_frequency = (int64_t) receiver->freq;
			*samples_per_packet = header->samples_per_packet;
		}
	}
	free(header);
	free(trailer);
	free(receiver);
	free(digitizer);
	free(extension);
	return result;

}

/**
 * send a SCPI command to the WSA
 *
 * @param wsa_handle - A handle containing the current socket information.
 * @param command - The command to be sent to the WSA.
 * @return 0 on success, or a negative number on error.
 */
__declspec(dllexport) int16_t __stdcall wsaSetSCPI(int64_t wsa_handle,  LPSTR command) 
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev = &wsa_dev;
	int16_t result;
	int32_t data_sock;
	int32_t cmd_sock;
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);
	result = wsa_send_scpi(dev, command);
	
	return result;
}

/**
 * Query a command
 *
 * @param wsa_handle - A handle containing the current socket information.
 * @param command - The command to be sent to the WSA.
 * @return A string containing the returned query.
 */
__declspec(dllexport) LPSTR __stdcall wsaGetSCPI(int64_t wsa_handle, LPSTR command)
{
	// declare the wsa structure
	struct wsa_device wsa_dev;

	// declare a pointer to the structure
	struct wsa_device *dev;
	dev = &wsa_dev;

	// create a static buffer to contain SCPI response
	char static buffer[512];

	// result of SCPI query
	int16_t result;

	// store the WSA sockets
	int32_t data_sock;
	int32_t cmd_sock;

	// parse the 64-bit value into the sockets
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);

	// initialize the device
	wsa_dev_init( cmd_sock, data_sock, dev);

	// query the SCPI command
	result = wsa_query_scpi(dev, command, buffer);

	// return the buffer with the query
	return buffer;

}

/**
 * Returns an error message corresponding to an error code
 *
 * @param wsa_handle - A handle containing the current socket information.
 * @param error_code - Error code
 * @return A string containing the error message corresponding to the error code
 */
__declspec(dllexport) LPSTR __stdcall wsaErrorMessage(int16_t error_code)
{
	static char buffer[255];
	sprintf_s(buffer, "%s\n", wsa_get_err_msg(error_code));
	return buffer;
}

/**
 * Returns the size of an array required to store spectral data for given
 * configuration
 *
 * @param wsa_handle - A handle containing the current socket information.
 * @param fstart - start frequency of sweep (Hz)
 * @param fstop - stop frequency of sweep (Hz)
 * @param rbw - rbw of sweep (Hz)
 * @param mode - mode of sweep (SH or SHN)
 * @param attenuator - the attenuator state (0=Off, 1=On)
 * @param sweep_size - Size of array
 * @return 0 on success, or a negative number on error.
 */	
__declspec(dllexport) int16_t __stdcall wsaGetSweepSize(int64_t wsa_handle, 
													uint64_t fstart, 
													uint64_t fstop, 
													uint32_t rbw, 
													LPSTR mode,
													int32_t attenuator,
													uint32_t *sweep_size) 
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev = &wsa_dev;
	struct wsa_power_spectrum_config *pscfg;
	struct wsa_sweep_device wsa_Sweep_Device;
	struct wsa_sweep_device *wsa_sweep_dev = &wsa_Sweep_Device;
	int16_t result = 5;
	int32_t data_sock;
	int32_t cmd_sock;

	// initialize wsa structure

	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);

	// create the sweep device
	wsa_sweep_dev = wsa_sweep_device_new(dev);

	// set the attenuator
	wsa_sweep_device_set_attenuator(wsa_sweep_dev, attenuator);
	// allocate memory for our ffts to go in
	result=  wsa_power_spectrum_alloc(wsa_sweep_dev, fstart, fstop, rbw, mode, &pscfg);
	if (result < 0)
	{
		wsa_power_spectrum_free(pscfg);
		wsa_sweep_device_free(wsa_sweep_dev);
		return result;
	}
	wsa_configure_sweep(wsa_sweep_dev, pscfg);
	*sweep_size = pscfg->buflen;
	wsa_power_spectrum_free(pscfg);
	wsa_sweep_device_free(wsa_sweep_dev);
	return result;

}

/**
 * Capture spectral data based on given configuration
 * Note: Call wsaGetSweepSize to determine the array size required to
 * store the result spectral data
 *
 * @param wsa_handle - A handle containing the current socket information.
 * @param fstart - start frequency of sweep (Hz)
 * @param fstop - stop frequency of sweep (Hz)
 * @param rbw - rbw of sweep (Hz)
 * @param mode - mode of sweep (SHN)
 * @param spectral_data - Array to hold the spectral data
 * @return 0 on success, or a negative number on error.
 */
__declspec(dllexport) int16_t __stdcall wsaCaptureSpectrum(int64_t wsa_handle, 
														uint64_t fstart, 
														uint64_t fstop, 
														uint32_t rbw, 
														LPSTR mode,
													    int32_t attenuator,
														float *spectral_data)
{

	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev =  &wsa_dev;
	struct wsa_power_spectrum_config *pscfg;
	struct wsa_sweep_device wsa_Sweep_Device;
	struct wsa_sweep_device *wsa_sweep_dev = &wsa_Sweep_Device;
	int16_t result;
	int16_t acq_status = 0;
	int32_t data_sock;
	int32_t cmd_sock;
	float *psbuf;
	// initialize wsa structure
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);

	// create the sweep device
	wsa_sweep_dev = wsa_sweep_device_new(dev);
	
	// set the attenuator
	wsa_sweep_device_set_attenuator(wsa_sweep_dev, attenuator);
	
	// allocate memory for our ffts to go in
	result = wsa_power_spectrum_alloc(wsa_sweep_dev, fstart, fstop, rbw, mode, &pscfg);
	if (result < 0)
	{
		wsa_power_spectrum_free(pscfg);
		wsa_sweep_device_free(wsa_sweep_dev);
		return result;
	}
	
	// capture power spectrum
	wsa_capture_power_spectrum(wsa_sweep_dev, pscfg, &psbuf);
	
	for (uint32_t i = 0; i < pscfg->buflen; i ++){
		 spectral_data[i] = pscfg->buf[i];
	}

	wsa_power_spectrum_free(pscfg);
	wsa_sweep_device_free(wsa_sweep_dev);
	return 0;

}
/**
 * Get the required buffer size to store the spectral data
 *
 * @param samples_per_packet- The sample size of the time domain data
 * @param stream_id - The stream id of the time domain data
 * @param array_size - A pointer to store the size of the array
 * @return 0 on success, or a negative number on error.
 */
__declspec(dllexport) int16_t __stdcall wsaGetFFTSize(int32_t samples_per_packet,
														uint32_t stream_id, 
														int32_t *array_size)
{
	int16_t result = 0;
	result = wsa_get_fft_size(samples_per_packet, stream_id, array_size);
	return result;
}

/**
 * Compute the FFT of the given time domain data
 * @param samples_per_packet - A pointer to store the number of samples
 * @param array_size - The size of the FFT buffer
 * @param stream_id - The IF data packet's stream id
 * @param reference_level - The reference level to calibrate the spectral data (dBm)
 * @param spectral_inversion - A pointer to to store the spectral inversion state
 * @param i16_data - A pointer to a 16-bit buffer to hold the 16-bit idata
 * @param q16_data - A pointer to a 16-bit buffer to hold the 16-bit qdata
 * @param i32_data - A pointer to a 32-bit buffer to hold the 32-bit idata
 * @param fft_buffer - A pointer to store the spectral data
 * @return 0 on success or a negative value on error
 */
__declspec(dllexport) int16_t __stdcall wsaComputeFFT(int32_t samples_per_packet,
													int32_t fft_size,
													uint32_t stream_id,
													int16_t reference_level,
													uint8_t spectral_inversion,
													int16_t * i16_buffer,
													int16_t * q16_buffer,
													int32_t * i32_buffer,
													float * fft_buffer)
{
	int16_t result = 0;
	result = wsa_compute_fft(samples_per_packet,
						fft_size,
						stream_id,
						reference_level,
						spectral_inversion,
						i16_buffer,
						q16_buffer,
						i32_buffer,
						fft_buffer);
	return result;
}

/**
 * Find the peak value within the specified range
 * Note you must set the sample size, and acquire read status before 
 * calling this function
 * @param wsa_handle - A handle containing the current socket information.
 * @fstart- An unsigned 64-bit integer containing the start frequency
 * @fstop - An unsigned 64-bit integer containing the stop frequency
 * @rbw - A 64-bit integer containing the RBW value of the captured data (in Hz)
 * @mode - A string containing the mode for the measurement
 * @ref_offset - A float used to apply any additional offset such as cable loss (dBm, positive value for gain, negative value for loss)
 * @attenuator - An integer to hold the 20dB attenuator's state (0 = on, 1 = off)
 * @peak_freq - An unsigned 64-bit integer to store the frequency of the peak(in Hz)
 * @peak_power - A flloating point pointer to store the power level of the peak (in dBm)
 *
 * @return 0 on success or a negative value on error
 */
__declspec(dllexport) int16_t __stdcall wsaPeakFind(int64_t wsa_handle,
					uint64_t fstart, 
					uint64_t fstop, 
					uint32_t rbw, 
					char *mode,
					float ref_offset,
					int32_t attenuator,
					uint64_t *peak_freq,
					float *peak_power)
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev =  &wsa_dev;
	int16_t result;
	int32_t data_sock;
	int32_t cmd_sock;
	uint64_t freq;
	float power;
	// initialize wsa structure
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);

	result = peak_find(dev, 
					fstart, 
					fstop, 
					rbw, 
					mode,
					attenuator,
					&freq,
					&power);
	*peak_power = power + ref_offset;
	*peak_freq = freq;
	return result;
}

/**
 * Calculate the channel power within a specific range
 * @param wsa_handle - A handle containing the current socket information.
 * @fstart- An unsigned 64-bit integer containing the start frequency
 * @fstop - An unsigned 64-bit integer containing the stop frequency
 * @rbw - A 64-bit integer containing the RBW value of the captured data (in Hz)
 * @mode - A string containing the mode for the measurement
 * @ref_offset - A float used to apply any additional offset such as cable loss (dBm)
 * @attenuator - An integer to hold the 20dB attenuator's state (0 = on, 1 = off)
 * @channel_power - A flloating point pointer to store the channel power(in dBm)
 *
 * @return 0 on success or a negative value on error
 */
__declspec(dllexport) int16_t __stdcall wsaChannelPower(int64_t wsa_handle,
					uint64_t fstart, 
					uint64_t fstop, 
					uint32_t rbw, 
					char *mode,
					float ref_offset,
					int32_t attenuator,
					float *channel_power)
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev =  &wsa_dev;
	int16_t result;
	int32_t data_sock;
	int32_t cmd_sock;
	float power;

	// initialize wsa structure
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);

	result = calculate_channel_power(dev, 
								fstart, 
								fstop, 
								rbw, 
								mode,
								attenuator,
								&power);
	*channel_power = power + ref_offset;
	return result;
}

/**
 * Calculate the occupied bandwidth
 * @param wsa_handle - A handle containing the current socket information.
 * @fstart- An unsigned 64-bit integer containing the start frequency
 * @fstop - An unsigned 64-bit integer containing the stop frequency
 * @rbw - A 64-bit integer containing the RBW value of the captured data (in Hz)
 * @occupied_percentage - How much of the channel power percentage should be in the occupied bandwidth
 * @mode - A string containing the mode for the measurement
 * @attenuator - An integer to hold the 20dB attenuator's state (0 = on, 1 = off)
 * @occupied_bandwidth -An unsigned 64-bit pointer to hold the frequency (MHz)
 *
 * @return 0 on success or a negative value on error
 */
__declspec(dllexport) int16_t __stdcall wsaOccupiedBandwidth(int64_t wsa_handle,
					uint64_t fstart, 
					uint64_t fstop, 
					uint32_t rbw, 
					float occupied_percentage,
					char *mode,
					int32_t attenuator,
					uint64_t *occupied_bw)
{
	struct wsa_device wsa_dev;
	struct wsa_device *dev;
	dev =  &wsa_dev;
	int16_t result;
	int32_t data_sock;
	int32_t cmd_sock;
	uint64_t freq;
	// initialize wsa structure
	parse_wsa_sockets(wsa_handle, &cmd_sock, &data_sock);
	wsa_dev_init( cmd_sock, data_sock, dev);

	result = calculate_occupied_bandwidth(dev, 
					fstart, 
					fstop, 
					rbw, 
					occupied_percentage,
					mode,
					attenuator,
					&freq);
	*occupied_bw= freq;
	return result;
}

// internal function used to reinitialize the R5500 device
void wsa_dev_init(int32_t cmd_sock, int32_t data_sock, struct wsa_device *dev)
{

	dev->sock.cmd = cmd_sock;
	dev->sock.data = data_sock;
	strcpy_s(dev->descr.intf_type, "TCPIP");

	// Initialized with "null" constants
	dev->descr.inst_bw = 125ULL * MHZ;
	dev->descr.max_sample_size = 65520;
	dev->descr.max_tune_freq = WSA_5000427_MAX_FREQ * MHZ;
	dev->descr.min_tune_freq = WSA_5000_MIN_FREQ;
	dev->descr.freq_resolution = WSA_5000_FREQRES;
	dev->descr.max_sample_size = WSA_MAX_CAPTURE_BLOCK;
	dev->descr.inst_bw = (uint64_t) WSA_IBW;
	dev->descr.max_decimation = WSA_MAX_DECIMATION;
	dev->descr.min_decimation = WSA_MIN_DECIMATION;

	// use the global variable to keep track of the unit model
	if (R5500_UNIT == 1){
		strcpy_s(dev->descr.prod_model, R5500);
		sprintf_s(dev->descr.dev_model, R5500427);
	
	}else{
		strcpy_s(dev->descr.prod_model, WSA5000);
		sprintf_s(dev->descr.dev_model, WSA5000427);
	}
	strcpy_s(dev->descr.fw_version, "4.3.6");
		
	
}


// function used to copy IQ data into proper buffers
void copy_iq_data(int16_t *i16_buffer, 
					int16_t *q16_buffer, 
					int32_t *i32_buffer, 
					int16_t i16_data[MAX_SAMPLES_PER_PACKET], 
					int16_t q16_data[MAX_SAMPLES_PER_PACKET],
					int32_t i32_data[MAX_SAMPLES_PER_PACKET], 
					uint32_t stream_id, 
					int32_t sample_size)
{
	if (stream_id == I16Q16_DATA_STREAM_ID)
	{
		for (int i = 0; i <= sample_size; i++)
		{
			i16_data[i] = i16_buffer[i];
			q16_data[i] = q16_buffer[i];
		}
	}
	else if (stream_id == I32_DATA_STREAM_ID)
	{
		for (int i = 0; i <= sample_size; i++)
			i32_data[i] = i32_buffer[i];

	}
	else if (stream_id == I16_DATA_STREAM_ID)
	{
		for (int i = 0; i <= sample_size; i++)
			i16_data[i] = i16_buffer[i];
	}
}

// internal function to recreate the sockets
void parse_wsa_sockets(int64_t wsa_handle, int32_t *cmd_sock, int32_t *data_sock)
{
	*cmd_sock = (int32_t) (wsa_handle >> 32);
	*data_sock = (int32_t) (wsa_handle & 0x0000ffff);
}